LEFT_FRONT_WHEEL_SPEED_SENSOR = "lfwss"
RIGHT_FRONT_WHEEL_SPEED_SENSOR = "rfwss"
LEFT_REAR_WHEEL_SPEED_SENSOR = "lrwss"
RIGHT_REAR_WHEEL_SPEED_SENSOR = "rrwss"
MIDDLE_REAR_WHEEL_SPEED_SENSOR = "mrwss"
